#
# basecgi.py - Base CGI module.
# Copyright (c) 2005 shinGETsu Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: basecgi.py 990 2005-05-15 03:03:59Z fuktommy $
#

import os
import sys

import config

__version__ = "$Revision: 990 $"


class CGI:

    """Base CGI class.

    start(): start the CGI.
    run(): main routine for CGI.

    """

    def __init__(self,
                 stdin=sys.stdin,
                 stdout=sys.stdout,
                 stderr=sys.stderr,
                 environ=os.environ):
        self.stdin = stdin
        self.stdout = stdout
        self.stderr = stderr
        self.environ = environ

    def start(self):
        """Start the CGI."""
        import socket
        try:
            self.run()
        except (IOError, socket.error, socket.timeout), strerror:
            self.stderr.write("%s: %s\n" %
                              (self.environ["REMOTE_HOST"], strerror))

    def run(self):
        """Main routine for CGI."""
        pass

    def path_info(self):
        """Parse PATH_INFO."""
        if "PATH_INFO" in self.environ:
            path = self.environ["PATH_INFO"]
            if path.startswith("/"):
                path = path[1:]
        else:
            path = ""
        return path

    def header(self, content="text/plain", addtional=None):
        self.stdout.write("Content-Type: " + content + "\n")
        if addtional is not None:
            for k in addtional:
                self.stdout.write(k + ": " + str(addtional[k]) + "\n")
        self.stdout.write("\n")
